@@ -2,6 +2,7 @@
 #'
 #' @inheritParams map_if
 #' @inheritParams map
+#' @return A vector the same type as `.x`.
 #' @export
 #' @examples
 #' pos <- function(x) x >= 0
@@ -13,6 +14,7 @@
 #' tail_while(0:10, big)
 head_while <- function(.x, .p, ...) {
   # Find location of first FALSE
+  .p <- as_predicate(.p, ..., .mapper = TRUE)
   loc <- detect_index(.x, negate(.p), ...)
   if (loc == 0) return(.x)
 
@@ -22,8 +24,9 @@ head_while <- function(.x, .p, ...) {
 #' @export
 #' @rdname head_while
 tail_while <- function(.x, .p, ...) {
+  .p <- as_predicate(.p, ..., .mapper = TRUE)
   # Find location of last FALSE
-  loc <- detect_index(.x, negate(.p), ..., .right = TRUE)
+  loc <- detect_index(.x, negate(.p), ..., .dir = "backward")
   if (loc == 0) return(.x)
 
   .x[-seq_len(loc)]
